@@ -25,6 +25,7 @@ from eqcorrscan.core.match_filter.family import Family
 from eqcorrscan.core.match_filter.template import Template
 from eqcorrscan.utils.plotting import plot_repicked
 
+show_interp_deprec_warning = True
 
 Logger = logging.getLogger(__name__)
 
@@ -43,14 +44,19 @@ class LagCalcError(Exception):
         return 'LagCalcError: ' + self.value
 
 
-def _xcorr_interp(ccc, dt):
+def _xcorr_interp(ccc, dt, resample_factor=10, use_new_resamp_method=False,
+                  **kwargs):
     """
-    Interpolate around the maximum correlation value for sub-sample precision.
+    Resample correlation-trace and check if there is a better CCC peak for
+    sub-sample precision.
 
     :param ccc: Cross-correlation array
     :type ccc: numpy.ndarray
     :param dt: sample interval
     :type dt: float
+    :param resample_factor:
+        Factor for upsampling CC-values (only for use_new_resamp_method=True)
+    :type resample_factor: int
 
     :return: Position of interpolated maximum in seconds from start of ccc
     :rtype: float
@@ -59,6 +65,32 @@ def _xcorr_interp(ccc, dt):
         cc = ccc[0]
     else:
         cc = ccc
+
+    # New method with resampling - make this the default in a future version
+    if use_new_resamp_method:
+        cc_resampled = scipy.signal.resample(cc, len(cc) * resample_factor + 1)
+        dt_resampled = dt / resample_factor
+        cc_t = np.arange(0, len(cc_resampled) * dt_resampled, dt_resampled)
+        peak_index = cc_resampled.argmax()
+        cc_peak = max(cc_resampled)
+
+        shift = cc_t[peak_index]
+        if (cc_peak < np.amax(cc) or cc_peak > 1.0 or
+                not 0 < shift < len(ccc) * dt):
+            # Sometimes the interpolation returns a worse result.
+            Logger.warning("Interpolation did not give an accurate result, "
+                           "returning maximum in data")
+            return np.argmax(ccc) * dt, np.amax(ccc)
+        return shift, cc_peak
+
+    # Otherwise use old interpolation method, but warn with deprcation message
+    # (but show it only once):
+    global show_interp_deprec_warning
+    if show_interp_deprec_warning:
+        Logger.warning(
+            'This method for interpolating cross-correlations is deprecated, '
+            'use a more robust method with use_new_resamp_method=True')
+        show_interp_deprec_warning = False
     # Code borrowed from obspy.signal.cross_correlation.xcorr_pick_correction
     cc_curvature = np.concatenate((np.zeros(1), np.diff(cc, 2), np.zeros(1)))
     cc_t = np.arange(0, len(cc) * dt, dt)
@@ -80,7 +112,7 @@ def _xcorr_interp(ccc, dt):
         Logger.debug(
             "Fewer than 5 samples selected for fit to cross correlation: "
             "{0}".format(num_samples))
-    coeffs, residual = scipy.polyfit(
+    coeffs, residual = np.polyfit(
         cc_t[first_sample:last_sample + 1],
         cc[first_sample:last_sample + 1], deg=2, full=True)[:2]
     # check results of fit
@@ -165,8 +197,13 @@ def _concatenate_and_correlate(streams, template, cores):
         cores=cores)
     # Re-order used_chans
     chan_order = chan_order[0]
-    for _used_chans in used_chans:
+    for i in range(len(used_chans)):
+        _used_chans = used_chans[i]
+        # Remove any channels that ended up not being used.
+        _used_chans = [c for c in _used_chans if c.channel in chan_order]
+        # Order the channels in the same way that they were correlated
         _used_chans.sort(key=lambda chan: chan_order.index(chan.channel))
+        used_chans[i] = _used_chans  # Put back in.
 
     # Reshape ccc output
     ccc_out = np.zeros((len(streams), len(chans),
@@ -183,9 +220,11 @@ def _concatenate_and_correlate(streams, template, cores):
 
 
 def xcorr_pick_family(family, stream, shift_len=0.2, min_cc=0.4,
+                      min_cc_from_mean_cc_factor=None,
                       horizontal_chans=['E', 'N', '1', '2'],
                       vertical_chans=['Z'], cores=1, interpolate=False,
-                      plot=False, plotdir=None):
+                      plot=False, plotdir=None, export_cc=False, cc_dir=None,
+                      **kwargs):
     """
     Compute cross-correlation picks for detections in a family.
 
@@ -202,6 +241,11 @@ def xcorr_pick_family(family, stream, shift_len=0.2, min_cc=0.4,
     :type min_cc: float
     :param min_cc:
         Minimum cross-correlation value to be considered a pick, default=0.4.
+    :type min_cc_from_mean_cc_factor: float
+    :param min_cc_from_mean_cc_factor:
+        If set to a value other than None, then the minimum cross-correlation
+        value for a trace is set individually for each detection based on:
+        min(detect_val / n_chans * min_cc_from_mean_cc_factor, min_cc).
     :type horizontal_chans: list
     :param horizontal_chans:
         List of channel endings for horizontal-channels, on which S-picks will
@@ -222,6 +266,13 @@ def xcorr_pick_family(family, stream, shift_len=0.2, min_cc=0.4,
     :type plotdir: str
     :param plotdir:
         Path to plotting folder, plots will be output here.
+    :type export_cc: bool
+    :param export_cc:
+        To generate a binary file in NumPy for every detection or not,
+        defaults to False
+    :type cc_dir: str
+    :param cc_dir:
+        Path to saving folder, NumPy files will be output here.
 
     :return: Dictionary of picked events keyed by detection id.
     """
@@ -245,17 +296,30 @@ def xcorr_pick_family(family, stream, shift_len=0.2, min_cc=0.4,
     for i, detection_id in enumerate(detection_ids):
         detection = [d for d in family.detections if d.id == detection_id][0]
         correlations = ccc[i]
+        if export_cc:
+            os.makedirs(cc_dir, exist_ok=True)
+            fname = f"{detection_id}-cc.npy"
+            np.save(os.path.join(cc_dir, f'{fname}'), correlations)
+            Logger.info(f"Saved correlation statistic to {fname} (lag_calc)")
         picked_chans = chans[i]
         detect_stream = detect_streams_dict[detection_id]
         checksum, cccsum, used_chans = 0.0, 0.0, 0
         event = Event()
+        if min_cc_from_mean_cc_factor is not None:
+            cc_thresh = min(abs(detection.detect_val / detection.no_chans
+                                * min_cc_from_mean_cc_factor),
+                            min_cc)
+            Logger.info('Setting minimum cc-threshold for detection %s to %s',
+                        detection.id, str(cc_thresh))
+        else:
+            cc_thresh = min_cc
         for correlation, stachan in zip(correlations, picked_chans):
             if not stachan.used:
                 continue
             tr = detect_stream.select(
                 station=stachan.channel[0], channel=stachan.channel[1])[0]
             if interpolate:
-                shift, cc_max = _xcorr_interp(correlation, dt=delta)
+                shift, cc_max = _xcorr_interp(correlation, dt=delta, **kwargs)
             else:
                 cc_max = np.amax(correlation)
                 shift = np.argmax(correlation) * delta
@@ -266,7 +330,7 @@ def xcorr_pick_family(family, stream, shift_len=0.2, min_cc=0.4,
             picktime = tr.stats.starttime + shift
             checksum += cc_max
             used_chans += 1
-            if cc_max < min_cc:
+            if cc_max < cc_thresh:
                 Logger.debug('Correlation of {0} is below threshold, not '
                              'using'.format(cc_max))
                 continue
@@ -357,7 +421,7 @@ def _prepare_data(family, detect_data, shift_len):
         length = round(length_samples) / family.template.samp_rate
         Logger.info("Setting length to {0}s to give an integer number of "
                     "samples".format(length))
-    prepick = shift_len
+    prepick = shift_len + family.template.prepick
     detect_streams_dict = family.extract_streams(
         stream=detect_data, length=length, prepick=prepick)
     for key, detect_stream in detect_streams_dict.items():
@@ -386,9 +450,10 @@ def _prepare_data(family, detect_data, shift_len):
 
 
 def lag_calc(detections, detect_data, template_names, templates,
-             shift_len=0.2, min_cc=0.4, horizontal_chans=['E', 'N', '1', '2'],
+             shift_len=0.2, min_cc=0.4, min_cc_from_mean_cc_factor=None,
+             horizontal_chans=['E', 'N', '1', '2'],
              vertical_chans=['Z'], cores=1, interpolate=False,
-             plot=False, plotdir=None):
+             plot=False, plotdir=None, export_cc=False, cc_dir=None, **kwargs):
     """
     Cross-correlation derived picking of seismic events.
 
@@ -419,6 +484,11 @@ def lag_calc(detections, detect_data, template_names, templates,
     :type min_cc: float
     :param min_cc:
         Minimum cross-correlation value to be considered a pick, default=0.4.
+    :type min_cc_from_mean_cc_factor: float
+    :param min_cc_from_mean_cc_factor:
+        If set to a value other than None, then the minimum cross-correlation
+        value for a trace is set individually for each detection based on:
+        min(detect_val / n_chans * min_cc_from_mean_cc_factor, min_cc).
     :type horizontal_chans: list
     :param horizontal_chans:
         List of channel endings for horizontal-channels, on which S-picks will
@@ -438,6 +508,13 @@ def lag_calc(detections, detect_data, template_names, templates,
         To generate a plot for every detection or not, defaults to False
     :param plotdir:
         Path to plotting folder, plots will be output here.
+    :type export_cc: bool
+    :param export_cc:
+        To generate a binary file in NumPy for every detection or not,
+        defaults to False
+    :type cc_dir: str
+    :param cc_dir:
+        Path to saving folder, NumPy files will be output here.
 
     :returns:
         Catalog of events with picks.  No origin information is included.
@@ -489,6 +566,10 @@ def lag_calc(detections, detect_data, template_names, templates,
             detections[n].id == output[m].resource_id
 
         if the output[m] is for the same event as detections[n].
+
+    .. note::
+        The correlation data that are saved to the binary files can be useful
+        to select an appropriate threshold for your data.
     """
     # First check that sample rates are equal for everything
     for tr in detect_data:
@@ -510,14 +591,16 @@ def lag_calc(detections, detect_data, template_names, templates,
             detections=template_detections,
             template=Template(
                 name=template_name, st=template,
-                samp_rate=template[0].stats.sampling_rate))
+                samp_rate=template[0].stats.sampling_rate, prepick=0.0))
         # Make a sparse template
         if len(template_detections) > 0:
             template_dict = xcorr_pick_family(
-                family=family, stream=detect_data,
-                min_cc=min_cc, horizontal_chans=horizontal_chans,
+                family=family, stream=detect_data, min_cc=min_cc,
+                min_cc_from_mean_cc_factor=min_cc_from_mean_cc_factor,
+                horizontal_chans=horizontal_chans,
                 vertical_chans=vertical_chans, interpolate=interpolate,
-                cores=cores, shift_len=shift_len, plot=plot, plotdir=plotdir)
+                cores=cores, shift_len=shift_len, plot=plot, plotdir=plotdir,
+                export_cc=export_cc, cc_dir=cc_dir, **kwargs)
             initial_cat.update(template_dict)
     # Order the catalogue to match the input
     output_cat = Catalog()
