@@ -23,7 +23,7 @@ import math
 from inspect import currentframe
 from scipy.signal import iirfilter, sosfreqz
 from collections import Counter
-from obspy import Trace
+from obspy import Stream, Trace
 from obspy.signal.invsim import simulate_seismometer as seis_sim
 from obspy.core.event import (
     Amplitude, Pick, WaveformStreamID, Origin, ResourceIdentifier)
@@ -366,7 +366,7 @@ def _pairwise(iterable):
 
 # Helpers for relative magnitude calculation
 
-def _get_pick_for_station(event, station, use_s_picks):
+def _get_pick_for_station(event, station, channel, use_s_picks):
     """
     Get the first reported pick for a given station.
 
@@ -380,7 +380,8 @@ def _get_pick_for_station(event, station, use_s_picks):
     :rtype: `obspy.core.event.Pick`
     :return: First reported pick for station
     """
-    picks = [p for p in event.picks if p.waveform_id.station_code == station]
+    picks = [p for p in event.picks if p.waveform_id.station_code == station
+             and p.waveform_id.channel_code == channel]
     if len(picks) == 0:
         Logger.info("No pick for {0}".format(station))
         return None
@@ -424,17 +425,20 @@ def _snr(tr, noise_window, signal_window):
 def _get_signal_and_noise(stream, event, seed_id, noise_window,
                           signal_window, use_s_picks):
     """
-    Get noise and signal amplitudes and signal standard deviation for an event
-    on a specific channel.
+    Get noise and signal RMS-amplitudes and signal standard deviation for an
+    event on a specific channel.
 
-    Noise amplitude is calculated as the RMS amplitude in the noise window,
-    signal amplitude is the maximum amplitude in the signal window.
+    (Until v.0.4.3, this function calculated noise amplitude as the RMS
+    amplitude of the noise window and signal amplitude as the maximum amplitude
+    in the signal window. This was changed to only RMS amplitudes to align it
+    with the methodology in Schaff & Richards 2014-paper.)
     """
     from eqcorrscan.core.template_gen import _rms
 
     station = seed_id.split('.')[1]
+    channel = seed_id.split('.')[3]
     pick = _get_pick_for_station(
-        event=event, station=station, use_s_picks=use_s_picks)
+        event=event, station=station, channel=channel, use_s_picks=use_s_picks)
     if pick is None:
         Logger.error("No pick for {0}".format(station))
         return None, None, None
@@ -442,9 +446,10 @@ def _get_signal_and_noise(stream, event, seed_id, noise_window,
     if len(tr) == 0:
         return None, None, None
     tr = tr[0]
-    noise_amp = _rms(tr.slice(
+    noise = tr.slice(
         starttime=pick.time + noise_window[0],
-        endtime=pick.time + noise_window[1]).data)
+        endtime=pick.time + noise_window[1]).data
+    noise_amp = _rms(noise)
     if np.isnan(noise_amp):
         noise_amp = None
     signal = tr.slice(
@@ -455,11 +460,12 @@ def _get_signal_and_noise(stream, event, seed_id, noise_window,
             pick.time + signal_window[0], pick.time + signal_window[1]))
         Logger.debug(tr)
         return noise_amp, None, None
-    return noise_amp, signal.max(), signal.std()
+    signal_amp = _rms(signal)
+    return noise_amp, signal_amp, signal.std()
 
 
 def relative_amplitude(st1, st2, event1, event2, noise_window=(-20, -1),
-                       signal_window=(-.5, 20), min_snr=5.0,
+                       signal_window=(-.5, 20), min_snr=1.5,
                        use_s_picks=False):
     """
     Compute the relative amplitudes between two streams.
@@ -475,7 +481,10 @@ def relative_amplitude(st1, st2, event1, event2, noise_window=(-20, -1),
     from st2.  The standard deviation of the amplitudes is computed in the
     signal window given. If the ratio of amplitudes between the signal window
     and the noise window is below `min_snr` then no result is returned for that
-    trace. Windows are computed relative to the first pick for that station.
+    trace. The SNR here is defined as the ratio of RMS-amplitudes of signal
+    and noise (equal to ratio of L2-norms of signal and noise, but normalized
+    for signal length). The Windows are computed relative to the first pick
+    for that station.
 
     If one stream has insufficient data to estimate noise amplitude, the noise
     amplitude of the other will be used.
@@ -502,11 +511,25 @@ def relative_amplitude(st1, st2, event1, event2, noise_window=(-20, -1),
         Note that noise and signal windows are relative to pick-times, so using
         an S-pick might result in a noise window including P-energy.
 
-    :rtype: dict
-    :return: Dictionary of relative amplitudes keyed by seed-id
+    :rtype: dict, dict, dict
+    :return:
+        Dictionary of relative amplitudes keyed by seed-id
+        Dictionary of signal-to-noise ratios for st1
+        Dictionary of signal-to-noise ratios for st2
     """
+    # keep input safe
+    event1 = event1.copy()
+    # sort out S-picks if not to be used
+    if not use_s_picks:
+        event1.picks = [p for p in event1.picks if p.phase_hint[0] != "S"]
+        st1 = Stream(
+            [tr for tr in st1.copy() if (tr.stats.station, tr.stats.channel) in
+             [(p.waveform_id.station_code, p.waveform_id.channel_code)
+              for p in event1.picks]])
     seed_ids = {tr.id for tr in st1}.intersection({tr.id for tr in st2})
     amplitudes = {}
+    snrs_1 = {}
+    snrs_2 = {}
     for seed_id in seed_ids:
         noise1, signal1, std1 = _get_signal_and_noise(
             stream=st1, event=event1, signal_window=signal_window,
@@ -535,7 +558,9 @@ def relative_amplitude(st1, st2, event1, event2, noise_window=(-20, -1),
         Logger.debug("Channel: {0} Relative amplitude: {1:.2f}".format(
             seed_id, ratio))
         amplitudes.update({seed_id: ratio})
-    return amplitudes
+        snrs_1.update({seed_id: snr1})
+        snrs_2.update({seed_id: snr2})
+    return amplitudes, snrs_1, snrs_2
 
 
 # Magnitude estimation functions
@@ -543,7 +568,7 @@ def relative_amplitude(st1, st2, event1, event2, noise_window=(-20, -1),
 def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
                        signal_window=(-.5, 20), min_snr=5.0, min_cc=0.7,
                        use_s_picks=False, correlations=None, shift=.2,
-                       return_correlations=False, weight_by_correlation=True):
+                       return_correlations=False, correct_mag_bias=True):
     """
     Compute the relative magnitudes between two events.
 
@@ -555,7 +580,8 @@ def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
 
     .. math::
 
-        \\Delta m = \\log{\\frac{std(tr2)}{std(tr1)}} \\times CC
+        \\Delta m = \\log{\\frac{std(tr2)}{std(tr1)}} + \\log{
+            \\frac{(1+\\frac{1}{snr_x^2})}{1+\\frac{1}{snr_y^2}}\\times CC}
 
     If you decide to use this function you should definitely read the paper
     to understand what you can use this for and cite the paper!
@@ -598,9 +624,11 @@ def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
     :type return_correlations: bool
     :param return_correlations:
         If true will also return maximum correlations as a dictionary.
-    :type weight_by_correlation: bool
-    :param weight_by_correlation:
-        Whether to weight the magnitude by the correlation or not.
+    :type correct_mag_bias: bool
+    :param correct_mag_bias:
+        Whether to correct for the magnitude-bias introduced by cc<1 and the
+        presence of noise (i.e., SNR << ∞). Without bias-correction, the
+        relative magnitudes are simple L2-norm-ratio relative magnitudes.
 
     :rtype: dict
     :return: Dictionary of relative magnitudes keyed by seed-id
@@ -613,7 +641,7 @@ def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
     if correlations is None:
         correlations = {}
         compute_correlations = True
-    relative_amplitudes = relative_amplitude(
+    relative_amplitudes, snrs_1, snrs_2 = relative_amplitude(
         st1=st1, st2=st2, event1=event1, event2=event2,
         noise_window=noise_window, signal_window=signal_window,
         min_snr=min_snr, use_s_picks=use_s_picks)
@@ -621,9 +649,11 @@ def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
         tr1 = st1.select(id=seed_id)[0]
         tr2 = st2.select(id=seed_id)[0]
         pick1 = _get_pick_for_station(
-            event=event1, station=tr1.stats.station, use_s_picks=use_s_picks)
+            event=event1, station=tr1.stats.station, channel=tr1.stats.channel,
+            use_s_picks=use_s_picks)
         pick2 = _get_pick_for_station(
-            event=event2, station=tr2.stats.station, use_s_picks=use_s_picks)
+            event=event2, station=tr2.stats.station, channel=tr2.stats.channel,
+            use_s_picks=use_s_picks)
         if compute_correlations:
             cc = correlate(
                 tr1.slice(
@@ -642,10 +672,14 @@ def relative_magnitude(st1, st2, event1, event2, noise_window=(-20, -1),
                 f"Correlation of {cc} less than {min_cc} for {seed_id}, "
                 "skipping.")
             continue
-        if not weight_by_correlation:
-            cc = 1.0
-        # Weight and add to relative_magnitudes
-        rel_mag = math.log10(amplitude_ratio) * cc
+        snr_x = snrs_1[seed_id]
+        snr_y = snrs_2[seed_id]
+        if not correct_mag_bias:
+            cc = snr_x = snr_y = 1.0
+        # Correct for CC and SNR-bias and add to relative_magnitudes
+        # This is equation 10 from Schaff & Richards 2014:
+        rel_mag = math.log10(amplitude_ratio) + math.log10(
+            math.sqrt((1 + 1 / snr_y**2) / (1 + 1 / snr_x**2)) * cc)
         Logger.info(f"Channel: {seed_id} Magnitude change {rel_mag:.2f}")
         relative_magnitudes.update({seed_id: rel_mag})
     if return_correlations:
@@ -988,7 +1022,7 @@ def svd_moments(u, s, v, stachans, event_list, n_svs=2):
 
     For more information see the paper by
     `Rubinstein & Ellsworth (2010).
-    <http://www.bssaonline.org/content/100/5A/1952.short>`_
+    <https://pubs.geoscienceworld.org/ssa/bssa/article/100/5A/1952/325099>`_
 
     :type u: list
     :param u:
